#!/usr/local/bin/cl -f
#
#This macro uses IRAF to find the first 
#and last pixel with a flux > 0; then it
#converts these two values in wavelength
#adding/subtracting 5 Armstrong.
#It NEEDS input.iraf file where it is stored
#the filename of the fits to be analysed;
#it OUTPUTS output_M01.iraf where the parameters
#SH and LO for DAOSPEC are saved.

noao
onedspec
images
imutil

{
string spectrum
string cont1
string cont2
string sing_p
string *input

real crval
real cdelt
real crpix
real crstart
real npixel
#real bpix
real wshift
real min0
real medc1
real medc2
real rcf
real flux0
real crlast

int ha_w1
int ha_w2
int ha_c1
int ha_c2
int ha_c3
int ha_c4
int npixel1
int npixel2
int first0
int first1
int skip

skip = 0
crval = INDEF
cdelt = INDEF
crpix = INDEF
npixel = INDEF
spectrum = INDEF
input = "input.iraf"
while(fscan(input,spectrum) !=EOF) {
}
if (spectrum=="INDEF"){
  print("IRAF: ...could not find all the needed information in input.iraf. Exiting.")
}
else {
  print("\nIRAF: using iraf script shlobpix.cl")
  print("IRAF: working on spectrum  ",spectrum)
  print("IRAF: looking for keyword CRVAL1, CDELT1, CRPIX1, NAXIS1 ...")
  #Keyword MUST be read singularly otherwise bugs happen
  hselect(spectrum,"CRVAL1",yes) | scan(crval)
  hselect(spectrum,"CRPIX1",yes) | scan(crpix)
  hselect(spectrum,"NAXIS1",yes) | scan(npixel)
  hselect(spectrum,"CDELT1",yes) | scan(cdelt)
  #hselect(spectrum,"BITPIX",yes) | scan(bpix)
  if(cdelt==INDEF){
    print("IRAF: could not find CDELT1 keyword in the header. Trying with CD1_1 ...")
    hselect(spectrum,"CD1_1",yes) | scan(cdelt)
    if(cdelt==INDEF || cdelt<=0){
      print("IRAF: could not find usable CD1_1 keyword.")
      skip=1
    }
    else if(cdelt>0){
      print("IRAF: CD1_1 keyword found. Adding CDELT1 keyword (equal to CD1_1) in the spectrum header.")
      hedit(spectrum,"CDELT1",cdelt,delete=no,addonly=yes,verify=no,update=yes)
    }
  }
  else if(cdelt<=0){
    print("IRAF: CDELT1 keyword found but is <= 0. Trying with CD1_1 ...")
    hselect(spectrum,"CD1_1",yes) | scan(cdelt)
    if(cdelt==INDEF || cdelt<=0){
      print("IRAF: could not find CD1_1 keyword.")
      skip=1
    }
    else if(cdelt>0){  
      print("IRAF: CD1_1 keyword found. Adding CDELT1 keyword (equal to CD1_1) in the spectrum header.")
      hedit(spectrum,"CDELT1",cdelt,delete=no,add=yes,verify=no,update=yes)
    }
  }
  if(crval==INDEF || crpix==INDEF || npixel==INDEF){
    print("IRAF: ...could not find all the needed information. Sorry.")
    skip=1
  }
  ;
  if(skip==0){
    print("IRAF: ... done!")
    crstart= crval-(crpix-1)*cdelt
    crlast = crval+(npixel*cdelt)
    crval  = crstart
  
    print("IRAF: CRVAL       ",crval)
    print("IRAF: CDELT       ",cdelt)
    print("IRAF: NPIXEL      ",npixel)
    print("IRAF: LAST PIXEL   ",crlast)
    #print("IRAF: BITPIX      ",bpix)
    
    ####### WAVELENGTH LIMITS  
    flux0 = 0
    i = 1    
    print("IRAF: ... now looking for starting wavelength ...")    
    while (flux0<=0)
    {
      sing_p=spectrum//"["//i//":"//i//"]"
      imstat(sing_p,fields="mean",format=no) | scan (flux0)
      i+=10
    }
    first0=crval+(i*cdelt)+5    
    flux0 = 0
    i = npixel    
    print("IRAF: ... now looking for ending wavelength ...")
    while (flux0<=0)
    {
      sing_p=spectrum//"["//i//":"//i//"]"
      imstat(sing_p,fields="mean",format=no) | scan (flux0)
      i-=10
    }
    first1=crval+(i*cdelt)-5
    print("IRAF: ... done!")
  
    print("\nIRAF: parameter SH and LO for DAOSPEC:")
    print("IRAF: SH  ",first0)
    print("IRAF: LO  ",first1)  
    print("sh=",first0, > "output_M01.iraf")
    print("lo=",first1, >> "output_M01.iraf")
    #print("bpix=",bpix, >> "output_M01.iraf")
  }
  else if (skip==1){
    print("IRAF: exiting the IRAF script shlobipix.cl. Analysis not complete for spectrum ",spectrum)
  }
}
logout
}
