#!/bin/bash

echo iraftsk: ----------   Module iraftsk is now running...   ----------


#CHECKING NEEDED FILE before going on
dir_fits=$WORK_DIR
outfile=checkfit.cl
infile=`grep -w output03 pipe.opt | cut -d= -f2`
if [ -z $infile ]; then
  echo "iraftsk: input file (module 03 output) is not defined in pipe.opt ..."
  echo "iraftsk: ...setting it as output03.dat"
  infile=output03.dat
fi
file_opt_miraf=${WORK_DIR}/${infile}

if [ -s $file_opt_miraf ]; then
  list=(`awk '!/^#/ {print $1}' $file_opt_miraf`)
    or=(`awk '!/^#/ {print $2}' $file_opt_miraf | cut -d= -f2`)
    sh=(`awk '!/^#/ {print $3}' $file_opt_miraf | cut -d= -f2`)
    lo=(`awk '!/^#/ {print $4}' $file_opt_miraf | cut -d= -f2`)			
  echo iraftsk: ...writing IRAF task...
  index=0
  rm -f $outfile
  for fits in ${list[@]}; do						
    skip=0					#variable used to skip file
    path=`echo $fits | sed -e "s/\/[^\/]*$//"`
    file=`echo $fits | sed -e "s/.*\///"`
    rootname=`echo $file | cut -d. -f1`  
    xmin=${sh[$index]}
    xmax=${lo[$index]}  
    if [ ${or[$index]} -ge 0 ]; then 
      if [ -f ${path}/${rootname}F.fits -a -f ${path}/${rootname}N.fits ]; then
        echo -e "specplot(spectra=\"${path}/${rootname}.fits,${path}/${rootname}C.fits\",fraction=0,xmin=$xmin,xmax=$xmax,ymin=-0.1,ymax=3,title=\"continuum for $rootname\",yscale=yes,step=0,scale=1,offset=0,autolayout=no,autoscale=no,cursor=\"cur.dat\")\n=gcur" >> $outfile
        echo -e "specplot(spectra=\"${path}/${rootname}N.fits,${path}/${rootname}F.fits\",fraction=0,xmin=$xmin,xmax=$xmax,ymin=-0.1,ymax=1.1,title=\"fit for $rootname\",yscale=yes,step=0,scale=1,offset=0,autolayout=no,autoscale=no,cursor=\"cur.dat\")\n=gcur" >> $outfile
      else
        echo iraftsk: Cannot find the normalized and/or fit spectra associated to ${path}/${fits}
      fi
    else 
      if [ -f ${path}/${rootname}F.fits -a -f ${path}/${rootname}.fits ]; then
        echo -e "specplot(spectra=\"${path}/${rootname}.fits,${path}/${rootname}F.fits\",fraction=0,xmin=$xmin,xmax=$xmax,ymin=-0.1,ymax=1.1,title=\"star: $rootname\",yscale=yes,step=0,scale=1,offset=0,autolayout=no,autoscale=no,cursor=\"cur.dat\")\n=gcur" >> $outfile
      else
        echo iraftsk: Cannot find the original and/or fit spectra associated to ${path}/${fits}
      fi
    fi
    ((index++))
  done 
  if [ -s $outfile ]; then
    echo -e "iraftsk: INSTRUCTIONS to use the IRAF task $outfile:
\ta- open an IRAF terminal
\tb- load the modules noao and onedspec to run specplot
\tc- launch the IRAF script:
\t\t cl<checkfit.cl
\td- when in the graphic window, use the capital letters X,Y,Z (to zoom-in), 
\t   P (to zoom-out), E (to expand) and the number 0 to redraw.
\t   For other commands see the IRAF help for cursor."
    echo -e ":color[2] 3
r
q" > cur.dat
  else
    echo iraftsk: The IRAF task $outfile is empty or does not exist. Something went wrong.
  fi
else
  echo iraftsk: I cannot find the output file of module 3 $file_opt_m3.
  echo iraftsk: Exiting the module.
  exit 1
fi



