#!/bin/bash
# module 3
#This module uses the output of M02 (FITS_opt_M02.log).
#It computes the median value of the spectra with flag3=1
#It uses this value as an input value for the UVES spectra that still have flag3=0.
#Daospec is run one last time, with this value as a fixed FWHM.
#In case of GIRAFFE spectra one last DAOSPEC run is done for all the spectra
#fixing the fwhm to the median.
#The output file of M03 reports then the best parameters used for the last final
#run of DAOSPEC.

#VARIABLES TO BE FIXED BEFORE LAUNCHING PIPELINE

echo M03: ----------   module 03 is now running...   ----------

dir_fits=$WORK_DIR					#Working directory 
infile=`grep -w output02 pipe.opt | cut -d= -f2`	#Output file of module 2
majrevfile=`grep MAJfile.*= pipe.opt | cut -d= -f2`     #Major reviews file

if [ -z $infile ]; then
  echo "M03: input file (module 02 output) is not defined in pipe.opt ..."
  echo M03: ...setting it as output02.dat
  infile=output02.dat
fi
file_opt_m2=${dir_fits}/${infile}

if [ ! -f $file_opt_m2 ]; then
  echo M03: I cannot find the output file of module 2 $file_opt_m2.
  echo M03: Exiting the module.
  exit 1
fi

outfile=`grep -w output03 pipe.opt | cut -d= -f2`	#Output file of module 3
if [ -z $outfile ]; then
  echo M03: output file is not defined in pipe.opt ...
  echo ...setting it as output03.dat
  outfile=output03.dat
fi
file_opt_m3=${dir_fits}/${outfile}

if [ -z $majrevfile ]; then
  echo M03: major reviews file is not defined in pipe.opt ...
  majrevfile=majorrev.dat
  echo ...setting it as $majrevfile.
fi
majrevfile=${WORK_DIR}/$majrevfile

cd $dir_fits
rm -f median.tmp

#CHECKING DUPLICATES IN inupt file and spectra in common with output file
checkduplicate=`grep -v \# $file_opt_m2 | awk 'x[$1]++' | wc -l`
if [ $checkduplicate -gt 0 ]; then echo M03 file $file_opt_m2 contains duplicated spectra. Please correct it.; exit 1; fi
if [ -f $file_opt_m3 ]; then
  checkduplicate=`cat $file_opt_m2 $file_opt_m3 | grep -v \# | awk 'x[$1]++' | wc -l` 
  if [ $checkduplicate -gt 0 ]; then 
    echo M03: file $file_opt_m2 and $file_opt_m3 have spectra in common. 
    echo M03: This may crash the pipeline. Please correct it.; exit 1;
  fi
fi

echo "#" >> $file_opt_m3 

#Read the output of M02:
list=(`grep -v \# $file_opt_m2 | awk '{print $1}'`)
stype=(`grep -v \# $file_opt_m2 | awk '{print $21}'`)

if [ ${#list[@]} -eq 0 ]; then
  echo M03: No usable spectra in $file_opt_m2. Exiting the module.
  exit 1
fi

#this command listen to the signal CTLR+C and force the script to stop with exit status 1
trap 'echo M03: ...forced to exit.; exit 1' 2	

iu=0; ig=0; skip=0; index=0
for fits in ${list[@]}; do
  if [ ${stype[$index]} == 'U' ]; then
    listU[$iu]=${list[$index]}
    orU[$iu]=`grep $fits $file_opt_m2 | awk '{print $2}'`
    shU[$iu]=`grep $fits $file_opt_m2 | awk '{print $3}'`
    loU[$iu]=`grep $fits $file_opt_m2 | awk '{print $4}'`
    leU[$iu]=`grep $fits $file_opt_m2 | awk '{print $5}'`
    riU[$iu]=`grep $fits $file_opt_m2 | awk '{print $6}'`
    reU[$iu]=`grep $fits $file_opt_m2 | awk '{print $7}'`
    miU[$iu]=`grep $fits $file_opt_m2 | awk '{print $8}'`
    maU[$iu]=`grep $fits $file_opt_m2 | awk '{print $9}'`
    veU[$iu]=`grep $fits $file_opt_m2 | awk '{print $10}'`
    fiU[$iu]=`grep $fits $file_opt_m2 | awk '{print $11}'`
    scU[$iu]=`grep $fits $file_opt_m2 | awk '{print $12}'`
    crU[$iu]=`grep $fits $file_opt_m2 | awk '{print $13}'`
    waU[$iu]=`grep $fits $file_opt_m2 | awk '{print $14}'`
    smU[$iu]=`grep $fits $file_opt_m2 | awk '{print $15}'`
    baU[$iu]=`grep $fits $file_opt_m2 | awk '{print $16}'`
    fwU[$iu]=`grep $fits $file_opt_m2 | awk '{print $17}'`
    fwiU[$iu]=`grep $fits $file_opt_m2 | awk '{print $17}'`
    flag1U[$iu]=`grep $fits $file_opt_m2 | awk '{print $18}'`
    flag2U[$iu]=`grep $fits $file_opt_m2 | awk '{print $19}'`
    flag3U[$iu]=`grep $fits $file_opt_m2 | awk '{print $20}'`
    ((iu++))
  elif [ ${stype[$index]} == 'G' ]; then
    listG[$ig]=${list[$index]}
    orG[$ig]=`grep $fits $file_opt_m2 | awk '{print $2}'`
    shG[$ig]=`grep $fits $file_opt_m2 | awk '{print $3}'`
    loG[$ig]=`grep $fits $file_opt_m2 | awk '{print $4}'`
    leG[$ig]=`grep $fits $file_opt_m2 | awk '{print $5}'`
    riG[$ig]=`grep $fits $file_opt_m2 | awk '{print $6}'`
    reG[$ig]=`grep $fits $file_opt_m2 | awk '{print $7}'`
    miG[$ig]=`grep $fits $file_opt_m2 | awk '{print $8}'`
    maG[$ig]=`grep $fits $file_opt_m2 | awk '{print $9}'`
    veG[$ig]=`grep $fits $file_opt_m2 | awk '{print $10}'`
    fiG[$ig]=`grep $fits $file_opt_m2 | awk '{print $11}'`
    scG[$ig]=`grep $fits $file_opt_m2 | awk '{print $12}'`
    crG[$ig]=`grep $fits $file_opt_m2 | awk '{print $13}'`
    waG[$ig]=`grep $fits $file_opt_m2 | awk '{print $14}'`
    smG[$ig]=`grep $fits $file_opt_m2 | awk '{print $15}'`
    baG[$ig]=`grep $fits $file_opt_m2 | awk '{print $16}'`
    fwG[$ig]=`grep $fits $file_opt_m2 | awk '{print $17}'`
    fwiG[$ig]=`grep $fits $file_opt_m2 | awk '{print $17}'`
    flag1G[$ig]=`grep $fits $file_opt_m2 | awk '{print $18}'`
    flag2G[$ig]=`grep $fits $file_opt_m2 | awk '{print $19}'`
    flag3G[$ig]=`grep $fits $file_opt_m2 | awk '{print $20}'`
    ((ig++))
  else
    echo "M03: $fits is not GIRAFFE or UVES type. Skipping it."
    ((skip++))
  fi
  ((index++))
done
echo "M03: I found $iu UVES spectra and $ig GIRAFFE spectra in $file_opt_m2. $skip spectra will be skipped."

nbspectra=`awk -v PIP="$iu" -v PLU="$ig" 'BEGIN {print PIP+PLU}'`

if [ $nbspectra -eq 0 ]; then
  echo M03: No usable spectra in $file_opt_m2. Exiting the module.
  exit 1
fi

#Calculate the median fwhm of the converged spectra
#If UVES spectra are found then playing with them
if [ $iu != 0 ]; then
  echo "M03: Now playing with UVES spectra ... "
  rm -f median.temp
  index=0; nbflag3U=0; notf=0; nfixU=0; notdao=0	
  for fits in ${listU[@]}; do
    if [ ${flag3U[$index]} -eq 1 -a ${flag1U[$index]} -eq 0 ]; then 
      echo ${fwU[$index]} | sed 's/fw=//g' >> median.temp
    elif [ ${flag3U[$index]} -eq 0 ]; then
      ((nbflag3U++))
    elif [ ${flag1U[$index]} -eq 1 ]; then
      ((nfixU++))
    else
      ((notf++))
    fi
    ((index++))
  done
  if [ $notf -eq $iu ]; then
    echo M03: No spectra in $file_opt_m2 converged during module 2 analysis. Exiting the module.
    exit 2
  elif [ $nfixU -eq $iu ]; then
    echo M03: All spectra have fixed FWHM. No stuff to do here. Copying the output of module 2 as output of module 3.
    cp $file_opt_m2 $file_opt_m3
  elif [ ! -s median.temp ]; then
    echo M03: Not possible to compute the median FWHM. Exiting the module.
    exit 2
  else
    
    medianfwhm=`sort -n median.temp | awk ' { a[i++]=$1; } END { x=int((i+1)/2); if (x < (i+1)/2) print (a[x-1]+a[x])/2; else print a[x-1]; }'`
    rm -f median.temp

    echo M03: The median FWHM for converged UVES spectra is $medianfwhm.

    if [ $nbflag3U -gt 0 ]; then echo M03: Now running Daospec with this value as fixed FWHM for not converged UVES spectra...; fi
    #Create daospec.opt file and run Daospec
    index=0	  
    for fits in ${listU[@]}; do
      path=`echo $fits | sed -e "s/\/[^\/]*$//"`
      file=`echo $fits | sed -e "s/.*\///"`
      rootname=`echo $fits | sed 's/.fits//g'`
      if [ ${flag3U[$index]} -eq 0 ]; then
  	#Set median as new input fw for the spectra with flag3=0, and fix the fw
  	fwU[$index]=fw=$medianfwhm
  	fiU[$index]=fi=1
  	sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M03: using fixed FWHM '$medianfwhm'.|' $majrevfile
  	cd $path
  	rm -f ${rootname}.daospec ${rootname}C.fits ${rootname}R.fits
  	echo -e ${orU[$index]} "\n"${shU[$index]} "\n"${loU[$index]} "\n"${leU[$index]} "\n"${riU[$index]} "\n"${reU[$index]} "\n"${miU[$index]} "\n"${maU[$index]} "\n"${veU[$index]} "\n"${fiU[$index]} "\n"${scU[$index]} "\n"${crU[$index]} "\n"${waU[$index]} "\n"${smU[$index]} "\n"${baU[$index]} "\n"${fwU[$index]} > daospec.opt
  	echo M03: Running Daospec on spectrum $fits...
  	echo "#DAOSPEC module 03 logfile" > ${rootname}_M03.log
  	cat daospec.opt >> ${rootname}_M03.log
daospec << END >> ${rootname}_M03.log				
  
  
$file
  
END
      fi
    
      #Creating output file *M03.daospec which contains the results
      #of the last DAOSPEC analysis with only the matched lines in it
      outdaos=${rootname}.daospec
      if [ ! -s $outdaos ]; then
  	echo M03: file $outdaos not found or empty. Changing flag3=${flag3U[$index]} to flag3=2 for spectrum $file. Please check again this file because something went wrong.
  	flag3U[$index]=2
  	((notdao++))
  	sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M03: Problem with DAOSPEC. Cannot find FWHM. Setting Flag3 to 2 and skipping further analysis.|' $majrevfile
      elif [ ${flag3U[$index]} -eq 0 ]; then
  	echo M03: ...done.
      fi
  
      #Now writing the daospec options used in the output file of module3
      echo -e "$fits ${orU[$index]} ${shU[$index]} ${loU[$index]} ${leU[$index]} ${riU[$index]} ${reU[$index]} ${miU[$index]} ${maU[$index]} ${veU[$index]} ${fiU[$index]} ${scU[$index]} ${crU[$index]} ${waU[$index]} ${smU[$index]} ${baU[$index]} ${fwU[$index]} ${flag1U[$index]} ${flag2U[$index]} ${flag3U[$index]}  U" >> $file_opt_m3
      _fi=`echo ${fwiU[$index]} | cut -d= -f2`
      _fo=`echo ${fwU[$index]} | cut -d= -f2`
      _temp=`awk -v PIP="$_fo" -v PLU="$_fi" 'function abs(x){return ((x < 0.0) ? -x : x)} BEGIN{FRAC=100*(PIP-PLU)/PLU;{printf "%i",abs(FRAC)}}'`
      if [ $_temp -gt 70 ]; then
  	sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M03: FWHM largely (more than 70%) changed from input value passing from '$_fi' to '$_fo'|' $majrevfile
      elif [ $_temp -ne 0 ]; then
  	sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM03: FWHM changed from '$_fi' to '$_fo'|' $majrevfile
      fi	  
      ((index++))
    #END of the FOR loop
    done
  fi
  echo M03: $nbflag3U on $iu UVES spectra had not converged FWHM from M02. 
  echo M03: $notf on $iu UVES spectra had not defined FWHM and were not analysed.
  if [ $notdao -gt 0 ]; then echo M03: $notdao spectra had no DAOSPEC output file. They are flagged with flag3=2. Please check them because something went wrong during the anlysis.; fi
fi

cd $dir_fits
rm -f median.temp

#If GIRAFFE spectra are found then playing with them
if [ $ig != 0 ]; then
  echo "M03: now playing with GIRAFFE spectra ... "
  index=0; nbflag3G=0; notf=0; nfixG=0; notdao=0	
  for fits in ${listG[@]}; do
    if [ ${flag3G[$index]} -eq 1 -a ${flag1G[$index]} -eq 0 ]; then 
      echo ${fwG[$index]} | sed 's/fw=//g' >> median.temp
    elif [ ${flag3G[$index]} -eq 0 ]; then
      ((nbflag3G++))
    elif [ ${flag1G[$index]} -eq 1 ]; then
      ((nfixG++))
    else
      ((notf++))
    fi
    ((index++))
  done
  if [ $notf -eq $ig ]; then
    echo M03: No spectra in $file_opt_m2 converged during module 2 analysis. Exiting the module.
    exit 2
  elif [ $nfixG -eq $ig ]; then
    echo M03: All spectra have fixed FWHM. No stuff to do here. Copying the output of module 2 as output of module 3.
    cp $file_opt_m2 $file_opt_m3
  elif [ ! -s median.temp ]; then
    echo M03: Not possible to compute the median FWHM. Exiting the module.
    exit 2
  else
    medianfwhm=`sort -n median.temp | awk ' { a[i++]=$1; } END { x=int((i+1)/2); if (x < (i+1)/2) print (a[x-1]+a[x])/2; else print a[x-1]; }'`
    rm -f median.temp

    echo M03: The median FWHM for GIRAFFE spectra is $medianfwhm.

    echo M03: Now running Daospec with this value as fixed FWHM for all GIRAFFE spectra...
    #Create daospec.opt file and run Daospec
    index=0;	  
    for fits in ${listG[@]}; do
      path=`echo $fits | sed -e "s/\/[^\/]*$//"`
      file=`echo $fits | sed -e "s/.*\///"`
      rootname=`echo $fits | sed 's/.fits//g'`
      #Set median as new input fw for the spectra with flag3=0, and fix the fw
      fwG[$index]=fw=$medianfwhm
      fiG[$index]=fi=1
      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M03: using fixed FWHM '$medianfwhm'.|' $majrevfile
      cd $path
      rm -f ${rootname}.daospec ${rootname}C.fits ${rootname}R.fits
      echo -e ${orG[$index]} "\n"${shG[$index]} "\n"${loG[$index]} "\n"${leG[$index]} "\n"${riG[$index]} "\n"${reG[$index]} "\n"${miG[$index]} "\n"${maG[$index]} "\n"${veG[$index]} "\n"${fiG[$index]} "\n"${scG[$index]} "\n"${crG[$index]} "\n"${waG[$index]} "\n"${smG[$index]} "\n"${baG[$index]} "\n"${fwG[$index]} > daospec.opt
      echo M03: Running Daospec on spectrum $fits...
      echo "#DAOSPEC module 03 logfile" > ${rootname}_M03.log
      cat daospec.opt >> ${rootname}_M03.log
daospec << END >> ${rootname}_M03.log				
  
  
$file
  
END
  
      #Creating output file *M03.daospec which contains the results
      #of the last DAOSPEC analysis with only the matched lines in it
      outdaos=${rootname}.daospec
      if [ ! -s $outdaos ]; then
  	echo M03: file $outdaos not found. Changing flag3=${flag3G[$index]} to flag3=2 for spectrum $file. Please check again this file because something went wrong.
  	flag3G[$index]=2
  	((notdao++))
  	sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M03: Problem with DAOSPEC. Cannot find FWHM. Setting Flag3 to 2 and skipping further analysis.|' $majrevfile
      else
  	echo M03: ...done.
      fi
      
      #Now writing the daospec options used in the output file of module3
      echo -e "$fits ${orG[$index]} ${shG[$index]} ${loG[$index]} ${leG[$index]} ${riG[$index]} ${reG[$index]} ${miG[$index]} ${maG[$index]} ${veG[$index]} ${fiG[$index]} ${scG[$index]} ${crG[$index]} ${waG[$index]} ${smG[$index]} ${baG[$index]} ${fwG[$index]} ${flag1G[$index]} ${flag2G[$index]} ${flag3G[$index]}  G" >> $file_opt_m3
      _fi=`echo ${fwiG[$index]} | cut -d= -f2`
      _fo=`echo ${fwG[$index]} | cut -d= -f2`
      _temp=`awk -v PIP="$_fo" -v PLU="$_fi" 'function abs(x){return ((x < 0.0) ? -x : x)} BEGIN{FRAC=100*(PIP-PLU)/PLU;{printf "%i",abs(FRAC)}}'`
      if [ $_temp -gt 70 ]; then
  	sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M03: FWHM largely (more than 70%) changed from input value passing from '$_fi' to '$_fo'|' $majrevfile
      else
  	sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM03: FWHM changed from '$_fi' to '$_fo'|' $majrevfile
      fi    
      ((index++))
    #END of the FOR loop
    done
  fi
  echo M03: $nbflag3G on $ig GIRAFFE spectra had not converged FWHM from M02. 
  echo M03: $notf on $ig GIRAFFE spectra had not defined FWHM and were not analysed.
  if [ $notdao -gt 0 ]; then echo M03: $notdao spectra had no DAOSPEC output file. They are flagged with flag3=2. Please check them because something went wrong during the anlysis.; fi
fi

echo M03: The best final DAOSPEC parameters for the spectra were written in file \"$file_opt_m3\".
echo M03: Please check the three columns after DAOSPEC parameter list \(flag1, flag2, and flag3\):
echo -e "\tflag1 -> if 1 fix fwhm was asked from the beginning of the analysis
\tflag2 -> if 1 radial velocity dispersion was considered consistent otherwise 0; if not 1 or 0 some problems were encountered during the analysis.
\tflag3 -> if 1 the spectrum had a converged FWHM; if 0 the FWHM was fixed to a median value of the spectra group; if not 0 or 1 there were some problems and the analysis is not complete for the spectrum ."

cd $dir_fits
echo M03: module 03 has ended its duty!

exit 0
