#!/usr/bin/env python
#Tristan Cantat-Gaudin, 26/02/2014.
#This script can be used after having run the DOOp.
#It uses the inputlist.dat file used as input by DOOp.
#and the products of the pipeline (especially those created by
#the module05). It helps visually checking the work of the pipeline
#To start it:
#> ./visualcheck.py
#or:
#> ./visualcheck.py inputlist_foo.dat

	#These are the possible commands you can type in the prompt:
	# ENTER		- go to the next spectrum of the list
	# goto X	- go to the Xth spectrum of the list (ex: goto 9)
	# find Y	- go to the spectrum whose name contains the string Y (ex: find uvl_018)
	#		  if there are several, it will list them (with their number in the list)
	# !comment	- write the name of the spectrum and your comment in the log
	#		  file "visualcheck.log".
	# back (or b)	- go to the previous spectrum
	# exit (or q)	- exit the program
	# help		- print this message

#Markers can be positioned on top of the identified lines.
#You can choose if the lines are read from the .iron or .daospec file
#by indidicating:     marklinesMode='iron'    or       marklinesMode='daospec'
marklinesMode='daospec'
#If you choose marklineMode='daospec', specify which elements you want, and the EW range.
#for instance:        marklineElements=[12.0,14.0,26.0,26.1,]     and      marklineRange=[20,140]
#write	marklinesElements=[]     (ie empty list) to keep ALL elements
#NB: the elements are read in the "MOOG format" (ie FeII is 26.1)
marklinesElements=[26.0,26.1]
#marklinesElements=[12.0,14.0,26.0,26.1]
marklinesRange=[20,140]
#
#Now you don't need to touch the rest. -----------------------------------------
#
markerstyle=['v','s','d','p','^','>','h','8','<']











# Function:
def plotDaospecResults(root):
	continuum = root+'C.fits'
	normalized = root+'N.fits'
	fitted = root+'F.fits'
	dao = root+'.daospec'
	moog = root+'.moog'
	iron = root+'.iron'
	daospecFailed=False #will be switched to True if the .daospec file is missing

	#read values from the .iron output:
	arr=[]
	countFeIlines=0
	countFeIIlines=0
	try:
		inpiron = open(iron,"r")
		for line in inpiron.readlines():
			toto = line.split()
			arr.append(toto)

		for line in arr:
			countFeIlines = countFeIlines + line[1].count('26.0')
			countFeIIlines = countFeIIlines + line[1].count('26.1')
		countFeIlines = str(countFeIlines)
		countFeIIlines = str(countFeIIlines)
		wavIronI = [float(el[0]) for el in arr if (el[1]=='26.0')]
		wavIronII = [float(el[0]) for el in arr if (el[1]=='26.1')]


	except IOError:
		countFeIlines = '-'
		countFeIIlines = '-'
		wavIronI = []
		wavIronII = []


	#read values from the .daospec output:
	arr=[]
	try:
		inpdao = open(dao,"r")
		for line in inpdao.readlines():
			toto = line.split()
			arr.append(toto)

		dao1stline = 'Rad. vel.='+arr[0][3]+', rms='+arr[0][6]+', fwhm='+arr[0][9]+' fitrms='+arr[1][7]+', '+countFeIlines+' FeI and '+countFeIIlines+' FeII lines.'
		#in the case marklinesMode='iron', read the identified lines from the .iron file:
		if marklinesMode=='iron':
			radVel=float(arr[0][3])
			#add radial velocity to get observed wavelengths:
			wavIronI=[el*(1+(radVel/299792.458)) for el in wavIronI] 
			wavIronII=[el*(1+(radVel/299792.458)) for el in wavIronII] 
			fluxI=[1.1 for el in wavIronI]
			fluxII=[1.1 for el in wavIronII]

		#in the case marklinesMode='daospec', read the identified lines from the .daospec file:
		elif marklinesMode=='daospec':
			if len(marklinesElements)==0:
				wavIronI = [float(el[0]) for el in arr[2:] if (len(el)>5) and (float(el[2])>marklinesRange[0]) and (float(el[2])<marklinesRange[1])]
				elementForPlot = [float(el[7]) for el in arr[2:] if (len(el)>5) and (float(el[2])>marklinesRange[0]) and (float(el[2])<marklinesRange[1])]
			else:
				wavIronI = [float(el[0]) for el in arr[2:] if (len(el)>5) and (float(el[7]) in marklinesElements) and (float(el[2])>marklinesRange[0]) and (float(el[2])<marklinesRange[1])]
				elementForPlot = [float(el[7]) for el in arr[2:] if (len(el)>5) and (float(el[7]) in marklinesElements) and (float(el[2])>marklinesRange[0]) and (float(el[2])<marklinesRange[1])]
				ewrange = [float(el[2]) for el in arr[2:] if (len(el)>5) and (float(el[7]) in marklinesElements) and (float(el[2])>marklinesRange[0]) and (float(el[2])<marklinesRange[1])]
			fluxI=[1.1 for el in wavIronI]
			wavIronII=[]	#these two are initialized as empty lists
			fluxII=[]	#but they are not actually used, everything is stored in "wavIronI"
		else:
			print 'PROBLEM!'
			print 'Check that the variable marklinesMode is set to \'iron\' or to \'daospec\' (in quotes)'
			print 'at the beginning of this script.'


	except IOError:
		daospecFailed=True
		dao1stline = ''



	# 1) build ORIGINAL spectrum array
	hdulist = pyfits.open(inputSpectrum)
	#hdulist.info()			#displays info about the content of the file
					#(what we use for Daospec has only ONE extension)
	#print hdulist[0].header	#to print the whole header!
	wave_base = hdulist[0].header['CRVAL1']	# Angstrom
	wave_step = hdulist[0].header['CD1_1']	# Angstrom
	fluxOr = hdulist[0].data
	waveobsOr = np.arange(wave_base, wave_base+len(fluxOr)*wave_step, wave_step)
	if len(waveobsOr) == len(fluxOr) + 1:
		waveobsOr = waveobsOr[:-1]
	hdulist.close()

	# 2) build CONTINUUM spectrum array
	try:
		hdulist = pyfits.open(continuum)
		wave_base = hdulist[0].header['CRVAL1']	
		wave_step = hdulist[0].header['CD1_1']	
		fluxCont = hdulist[0].data
		waveobsCont = np.arange(wave_base, wave_base+len(fluxCont)*wave_step, wave_step)
		if len(waveobsCont) == len(fluxCont) + 1:
			waveobsCont = waveobsCont[:-1]
		hdulist.close()
	except IOError:
		fluxCont = [0]
		waveobsCont = [0]


	# 3) build NORMALIZED spectrum array
	try:
		hdulist = pyfits.open(normalized)
		wave_base = hdulist[0].header['CRVAL1']	
		wave_step = hdulist[0].header['CD1_1']
		fluxNorm = hdulist[0].data
		waveobsNorm = np.arange(wave_base, wave_base+len(fluxNorm)*wave_step, wave_step)
		if len(waveobsNorm) == len(fluxNorm) + 1:
			waveobsNorm = waveobsNorm[:-1]
		hdulist.close()
	except IOError:
		fluxNorm = fluxOr
		waveobsNorm = waveobsOr
		
	# 4) build FITTED spectrum array
	try:
		hdulist = pyfits.open(fitted)
		wave_base = hdulist[0].header['CRVAL1']	
		wave_step = hdulist[0].header['CD1_1']
		fluxFit = hdulist[0].data
		waveobsFit = np.arange(wave_base, wave_base+len(fluxFit)*wave_step, wave_step)
		if len(waveobsFit) == len(fluxFit) + 1:
			waveobsFit = waveobsFit[:-1]
		hdulist.close()
	except IOError:
		daospecFailed=True

		

	### ------------- Plots
	plt.ion()
	fig=plt.figure('0')
	plt.clf()
	ax0=plt.subplot(1,1,1)
	ax1=plt.subplot(2,1,1)
	ax1.plot(waveobsOr, fluxOr, color='gray', label='observed spectrum')
	plt.xlabel('$\lambda$ [$\AA$]')
	plt.ylabel('flux')
	plt.title(dao1stline,size=12)
	if daospecFailed==False:
		plt.plot(waveobsCont, fluxCont, color='red', label='continuum (order '+str(order)+')')
		plt.xlim(min(waveobsOr),max(waveobsOr))
		plt.ylim(0,1.15*max(fluxCont))
	else:
		plt.xlim(min(waveobsOr),max(waveobsOr))
		plt.ylim(0,2*fluxOr[len(fluxOr)/2])
	plt.legend(loc="lower right",prop={'size':12})

	ax2=plt.subplot(2,1,2,sharex=ax1)
	if daospecFailed==False:
		ax2.plot(waveobsNorm, fluxNorm, color='gray', label='normalized')
		ax2.plot(waveobsFit, fluxFit, color='blue', label='Daospec fit')
		#two different ways of marking the lines ('iron' or 'daospec'):
		if marklinesMode=='iron':
			plt.plot(wavIronI, fluxI, 'bo')
			plt.plot(wavIronII, fluxII, 'g^')
		if marklinesMode=='daospec':
			if len(marklinesElements)==0: #in case ALL elements are required:
				els = plt.scatter(wavIronI, fluxI,marker='o',c=elementForPlot,cmap=mpl.cm.gist_rainbow)
				cbar=plt.colorbar(els,ax=ax2,orientation='horizontal',aspect=50)
				cbar.set_label('element', rotation=0,labelpad=-40)
			else:
				for t,zeta in enumerate(marklinesElements): #for each element/ion:
					#filter only lines of that element (and associated wav and ew)
					t_index=[i for i, x in enumerate(elementForPlot) if x == zeta]	#index of the lines of considerer element/ion
					t_elementForPlot=[elementForPlot[el] for el in t_index]
					t_wavIronI=[wavIronI[el] for el in t_index]
					t_ewrange=[ewrange[el] for el in t_index]
					t_fluxI=[fluxI[el] for el in t_index]
					if len(t_index)>0: #if there indeed are lines of this element:
						scatter = plt.scatter(t_wavIronI,t_fluxI,marker=markerstyle[t],c=t_ewrange,cmap=mpl.cm.gist_rainbow,linewidth='0',s=50,vmin=marklinesRange[0],vmax=marklinesRange[1],label=str(zeta))
				try:				
					#add a color bar for the EWs:
					cbar=plt.colorbar(scatter,ax=ax2,orientation='horizontal',aspect=50,cmap=mpl.cm.gist_rainbow)
					cbar.set_label('EW')
				except:
					pass

		plt.xlim(min(waveobsOr),max(waveobsOr))
		if (max(fluxNorm)<1.3):
			plt.ylim(0,1.05*max(fluxNorm))
		else:
			plt.ylim(0,1.3)
		plt.legend(loc="lower right",prop={'size':12})
	plt.xlabel('$\lambda$ [$\AA$]')
	plt.ylabel('normalized flux')
	plt.text(0.2,0.15,'NO DAOSPEC OUTPUT AVAILABLE',size=20,rotation=20,color='red')
	plt.suptitle(inputSpectrum+'        ('+str(counter)+'/'+str(len(inputSpectra))+')',size=14)
	plt.draw()	#better than plt.show() because it doesn't freeze the shell (if you have called plt.ion() before)





# ----------------- Program starts here. ----------------- #
try:
	import pyfits
except:
	print 'Problem importing PyFits, maybe module is missing.'
try:
	import numpy as np
except:
	print 'Problem importing Numpy, maybe module is missing.'
try:
	import matplotlib.pyplot as plt
	import matplotlib as mpl
except:
	print 'Problem importing Pyplot, maybe module is missing.'
import sys


#if no argument provided, use inputlist.dat by default
try:
	fileName = sys.argv[1]
except IndexError:
	fileName = 'inputlist.dat'

logFileName = "visualcheck.log"

#arr is the table that will receive the content of inputlist.dat
arr=[]
try:
	inp = open(fileName,"r")
	for line in inp.readlines():
		numbers = line.split()
		arr.append(numbers)
	inputSpectra = [el[0] for el in arr if not el[0].startswith('#')]
	orders = [int(el[1]) for el in arr if not el[0].startswith('#')]
except:
	sys.exit('Problem reading the input list \"'+fileName+'\". EXIT.')

#--------------------------- LOOP
#
#now let's display plots for each of the measured spectra:
counter=0
exitProgram=False

logFile = open(logFileName, "a")
logFile.write('#\n')
logFile.close()

while exitProgram==False:
	star = zip(inputSpectra,orders)[counter]
	counter = counter+1
	inputSpectrum = star[0]
	order=star[1]

	root = inputSpectrum.split('.fits')[0]

	plotDaospecResults(root)

	spam = raw_input('Spectrum '+str(counter)+'/'+str(len(inputSpectra))+'... ')


	#Here are the commands:
	if ('!' in spam):
		if (spam[0]=='!'):		
			try:
				logFile = open(logFileName, "a")
				comment=spam[1:]
				logFile.write(root+'\t'+comment+'\n')
				logFile.close()
				if (counter>0):
					counter = counter - 1
			except:
				if (counter>0):
					counter = counter - 2
				else:
					counter = counter - 1
	else:
		if (spam=='exit') or (spam=='q') or (spam=='Q'):
			sys.exit()
		if (spam=='back') or (spam=='b'):
			if (counter>0):
				counter = counter - 2
			else:
				counter = counter - 1
		if ('goto' in spam):
			try:
				counter = int(spam.split('goto')[1])-1
			except:
				counter = counter - 1
		if ('help' in spam):
			print	' These are the possible commands you can type in the prompt:'
			print	'   ENTER         - go to the next spectrum of the list'
			print	'   goto X        - go to the Xth spectrum of the list (ex: goto 9)'
			print	'   find Y        - go to the spectrum whose name contains the string Y (ex: find uvl_018)'
			print	'                   if there are several, it will list them (with their number in the list)'
			print	'   !comment      - write the name of the spectrum and your comment in the log'
			print	'		    file visualcheck.log.'
			print	'   back (or b)   - go to the previous spectrum'
			print	'   exit (or q)   - exit the program'
			print   '   help          - print this message'
			print   ''

		if ('find' in spam):
			try:
				toFind = spam.split('find ')[1]
				possible = [s for s in inputSpectra if toFind in s]
				if len(possible)>1:
					print '   Ambiguous. Possibilities are:'
					for i in range(len(possible)):
						print '   '+possible[i]+' ('+str(inputSpectra.index(possible[i])+1)+')'
					counter = counter - 1
				if len(possible)==0:
					print '   No such name in the list \"'+fileName+'\", sorry.'
					counter = counter - 1
				if len(possible)==1:
					counter = inputSpectra.index(possible[0])
			except:
				counter = counter - 1

	if counter>len(inputSpectra)-1:
		print 'Spectrum number',len(inputSpectra),'is the last of',fileName,'.'
		print 'Type q to exit.'
		counter = counter - 1



